!
!     EAM FORCE & ENERGY CALCULATION
!
!     PRB 68, art.024102 (2003)
!     R. R. Zope and Y. Mishin
!
!     Fast calculation using mesh data
!     
!     calculation of force, total energy, (d(mat) <- for extension)
!     New bookkeep algorithm
!     
!   Oct. 5, 2010
!   A bug in stress calculation for a small cell is fixed

      module eam_tial_function
      double precision, allocatable :: pairx(:),pairy(:),pairc(:,:),pairxn(:)
      double precision, allocatable :: denx(:),deny(:),denc(:,:),denxn(:)
      double precision, allocatable :: embedx(:),embedy(:),embedc(:,:),embedxn(:)
      double precision, allocatable :: spd2(:)
      integer, parameter :: mesh=1000000
      double precision :: pairTi(mesh),pairdTi(mesh)
      double precision :: denTi(mesh),dendTi(mesh)
      double precision :: embedTi(mesh),embeddTi(mesh)
      double precision :: pairAl(mesh),pairdAl(mesh)
      double precision :: denAl(mesh),dendAl(mesh)
      double precision :: embedAl(mesh),embeddAl(mesh)
      double precision :: pairTiAl(mesh),pairdTiAl(mesh)
      dimension DF(2), IOPT(2), EC(4)
      double precision :: pairminTi,pairmaxTi
      double precision :: denminTi,denmaxTi
      double precision :: embedminTi,embedmaxTi
      double precision :: pairminAl,pairmaxAl
      double precision :: denminAl,denmaxAl
      double precision :: embedminAl,embedmaxAl
      logical, save :: eam_tial_setup_done = .false.
      end module

      subroutine eam_tial
      use eam_tial_function
      use variables
      use constants
      use bookkeep_param
      use debug_variables
      implicit double precision (a-h,o-z)
      double precision :: rhob(n) ! rho-bar_i (density)
      double precision :: ki,kif
      integer :: istat

      istat=1

!  Functions setup
      if (.not.eam_tial_setup_done) then
         print *,'eam (TiAl) setup'
         call eam_tial_setup
      endif

! Cutoff radius set
      rcTi=pairmaxTi*ang
      rcAl=pairmaxAl*ang
      rcTial=pairmaxTial*ang
      rcTi2=rcTi**2
      rcAl2=rcAl**2
      rcTial2=rcTial**2
      rc=rcTi
      if (rc<rcAl) rc=rcAl
      if (rc<rcTial) rc=rcTial
!      print *,'Rc=',rc/ang
      rc2=rc**2

      rhob=0.0d0

!   Virial term reset
      virx=0.0d0; viry=0.0d0; virz=0.0d0
      dmat=0.0d0
      satom=0.0d0               ! Atomic stress

!   First Loop: Chg dns calc
      kns=0
      idbg=0
      do 4200 k=1,n
         kn=knum(k)
         if (kn.eq.0) go to 4200
         do 4300 jk=1,kn
            kns=kns+1
            j=idpt(kns,1)
            ix=idpt(kns,2); iy=idpt(kns,3); iz=idpt(kns,4)
            xx=rx(k)-rx(j)
            yy=ry(k)-ry(j)
            zz=rz(k)-rz(j)
            call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)
            if (istat == 1) then
               r2=xx**2+yy**2+zz**2
               if (r2.lt.rc2) then
                  r=sqrt(r2)
                  if (asp(j)=='Ti') then
                     ii=int(dble(mesh)*(r/ang-denminTi)/(denmaxTi-denminTi))
                     rhob(k)=rhob(k)+denTi(ii)
                  else if (asp(j)=='Al') then
                     ii=int(dble(mesh)*(r/ang-denminAl)/(denmaxAl-denminAl))
                     rhob(k)=rhob(k)+denAl(ii)
                  else
                     stop 'EAM error'
                  endif
                  if (j/=k) then
                     if (asp(k)=='Ti') then
                        ii=int(dble(mesh)*(r/ang-denminTi)/(denmaxTi-denminTi))
                        rhob(j)=rhob(j)+denTi(ii)
                     else if (asp(k)=='Al') then
                        ii=int(dble(mesh)*(r/ang-denminAl)/(denmaxAl-denminAl))
                        rhob(j)=rhob(j)+denAl(ii)
                     else
                        stop 'EAM error'
                     endif
                  endif
               endif
            endif
 4300    continue
 4200 continue

!     Calculation of potential energy
      do i=1,n
         if (asp(i)=='Ti') then
            ii=int(dble(mesh)*(rhob(i)-embedminTi)/(embedmaxTi-embedminTi))
            enpoteach(i)=embedTi(ii)*ev
         else if (asp(i)=='Al') then
            ii=int(dble(mesh)*(rhob(i)-embedminAl)/(embedmaxAl-embedminAl))
            enpoteach(i)=embedAl(ii)*ev
         else
            stop 'EAM error'
         endif
      enddo

!   Second Loop: Force calc
      fx(1:n)=0.0d0
      fy(1:n)=0.0d0
      fz(1:n)=0.0d0
      kns=0
      do 4500 i=1,n
         kn=knum(i)
         if (kn.eq.0) go to 4500
         do 4600 jk=1,kn
            kns=kns+1
            j=idpt(kns,1)
            ix=idpt(kns,2); iy=idpt(kns,3); iz=idpt(kns,4)
            xx=rx(i)-rx(j)
            yy=ry(i)-ry(j)
            zz=rz(i)-rz(j)
            call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)
            if (istat == 1) then
               r2=xx**2+yy**2+zz**2
               if (r2.lt.rc2) then
                  r=sqrt(r2)
                  if (asp(i)/=asp(j)) then
                     ii=int(dble(mesh)*(r/ang-pairminTial)/(pairmaxTial-pairminTial))
                     vij=pairTial(ii)*ev
                     dv=pairdTial(ii)
                  else if (asp(i)=='Ti') then
                     ii=int(dble(mesh)*(r/ang-pairminTi)/(pairmaxTi-pairminTi))
                     vij=pairTi(ii)*ev
                     dv=pairdTi(ii)
                  else if (asp(i)=='Al') then
                     ii=int(dble(mesh)*(r/ang-pairminAl)/(pairmaxAl-pairminAl))
                     vij=pairAl(ii)*ev
                     dv=pairdAl(ii)
                  endif
                  enpoteach(i)=enpoteach(i)+vij/2.0d0
                  vijsum=vijsum+vij/2.0d0
                  if (j/=i) then
                     enpoteach(j)=enpoteach(j)+vij/2.0d0
                     vijsum=vijsum+vij/2.0d0
                  endif

                  if (asp(i)=='Ti') then
                     ii=int(dble(mesh)*(rhob(i)-embedminTi)/(embedmaxTi-embedminTi))
                     df_i=embeddTi(ii)
                  else
                     ii=int(dble(mesh)*(rhob(i)-embedminAl)/(embedmaxAl-embedminAl))
                     df_i=embeddAl(ii)
                  endif
                  if (asp(j)=='Ti') then
                     jj=int(dble(mesh)*(rhob(j)-embedminTi)/(embedmaxTi-embedminTi))
                     df_j=embeddTi(jj)
                  else
                     jj=int(dble(mesh)*(rhob(j)-embedminAl)/(embedmaxAl-embedminAl))
                     df_j=embeddAl(jj)
                  endif

                  if (asp(i)=='Ti') then
                     ii=int(dble(mesh)*(r/ang-denminTi)/(denmaxTi-denminTi))
                     dr_i=dendTi(ii)
                  else
                     ii=int(dble(mesh)*(r/ang-denminAl)/(denmaxAl-denminAl))
                     dr_i=dendAl(ii)
                  endif

                  if (asp(j)=='Ti') then
                     jj=int(dble(mesh)*(r/ang-denminTi)/(denmaxTi-denminTi))
                     dr_j=dendTi(jj)
                  else
                     jj=int(dble(mesh)*(r/ang-denminAl)/(denmaxAl-denminAl))
                     dr_j=dendAl(jj)
                  endif

                  f=-(df_i*dr_j+df_j*dr_i+dv)*ev/ang

                  ki=f/r
                  if (j==i) ki=ki/2.0d0
                  kif=ki/2.0d0
                  fx(i)=fx(i)+ki*xx
                  fy(i)=fy(i)+ki*yy
                  fz(i)=fz(i)+ki*zz
                  fx(j)=fx(j)-ki*xx
                  fy(j)=fy(j)-ki*yy
                  fz(j)=fz(j)-ki*zz
                  satom(i,1,1)=satom(i,1,1)-kif*xx*xx
                  satom(i,1,2)=satom(i,1,2)-kif*yy*xx
                  satom(i,2,2)=satom(i,2,2)-kif*yy*yy
                  satom(i,1,3)=satom(i,1,3)-kif*zz*xx
                  satom(i,2,3)=satom(i,2,3)-kif*zz*yy
                  satom(i,3,3)=satom(i,3,3)-kif*zz*zz
                  satom(j,1,1)=satom(j,1,1)-kif*xx*xx
                  satom(j,1,2)=satom(j,1,2)-kif*yy*xx
                  satom(j,2,2)=satom(j,2,2)-kif*yy*yy
                  satom(j,1,3)=satom(j,1,3)-kif*zz*xx
                  satom(j,2,3)=satom(j,2,3)-kif*zz*yy
                  satom(j,3,3)=satom(j,3,3)-kif*zz*zz
               endif
            endif
 4600    continue
 4500 continue

!---Etot---
      enpot=0.0d0
      do i=1,n
         enpot=enpot+enpoteach(i)
      enddo

!!     Atomic stress
      do i=1,n
         satom(i,2,1)=satom(i,1,2)
         satom(i,3,1)=satom(i,1,3)
         satom(i,3,2)=satom(i,2,3)
      enddo
      dmat=0.0d0
! The sign of dmat is opposite to stress tensor's (dmat is the "virial" tensor)
      do i=1,n
         dmat(1:3,1:3)=dmat(1:3,1:3)-satom(i,1:3,1:3)
      enddo
      virx=dmat(1,1); viry=dmat(2,2); virz=dmat(3,3)

      satom=satom*dble(n)/v

!      write (10,*) 'EMT force calculation end.'
      end



!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!      
!     EAM SETUP
!     READ FUNCTIONS
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!      
      subroutine eam_tial_setup
      use eam_tial_function
      use constants
      implicit double precision(a-h,o-z)
      character (1) :: ichar
      logical :: alonly = .false.
      if (TRIM(potential_arg)=='ZopeAl') alonly = .true.
      
! Pair function
      do iii=1,3
         if (iii==1) open (10,file='pot/EAMPARAM.PRB68-024102.ti.pair')
         if (iii==2) then
            if (alonly) open (10,file='pot/EAMPARAM.PRB68-024102-Al.al.pair')
            if (.not.alonly) open (10,file='pot/EAMPARAM.PRB68-024102.al.pair')
         endif
         if (iii==3) open (10,file='pot/EAMPARAM.PRB68-024102.tial.pair')
         mesh_pair=0
 10      read (10,*,end=30) ichar !trial read
         if (ichar(:1)=='#') then
            go to 10            !skip a line if it is a comment line
         else
            backspace 10        !read again if it is not a comment line
         endif
 20      read (10,*,end=30,err=10) a1,a2
         mesh_pair=mesh_pair+1
         go to 20
 30      close (10)
         allocate(pairx(mesh_pair),pairy(mesh_pair),pairc(mesh_pair,3))
         
         if (iii==1) open (10,file='pot/EAMPARAM.PRB68-024102.ti.pair')
         if (iii==2) then
            if (alonly) open (10,file='pot/EAMPARAM.PRB68-024102-Al.al.pair')
            if (.not.alonly) open (10,file='pot/EAMPARAM.PRB68-024102.al.pair')
         endif
         if (iii==3) open (10,file='pot/EAMPARAM.PRB68-024102.tial.pair')
         mesh_pair=0
 15      read (10,*,end=35) ichar !trial read
         if (ichar(:1)=='#') then
            go to 15            !skip a line if it is a comment line
         else
            backspace 10        !read again if it is not a comment line
         endif
 25      read (10,*,end=35,err=15) a1,a2
         mesh_pair=mesh_pair+1
         if (mesh_pair==1) then
            if (iii==1) pairminTi=a1
            if (iii==2) pairminAl=a1
            if (iii==3) pairminTial=a1
         endif
         pairx(mesh_pair)=a1
         pairy(mesh_pair)=a2
         go to 25
 35      close (10)
         if (iii==1) pairmaxTi=a1
         if (iii==2) pairmaxAl=a1
         if (iii==3) pairmaxTial=a1
!     finer mesh set
         allocate(pairxn(mesh),spd2(mesh))
         if (iii==1) then
            x=pairminTi
            d=(pairmaxTi-pairminTi)/dble(mesh-1)
         else if (iii==2) then
            x=pairminAl
            d=(pairmaxAl-pairminAl)/dble(mesh-1)
         else
            x=pairminTial
            d=(pairmaxTial-pairminTial)/dble(mesh-1)
         endif
         do i=1,mesh
            pairxn(i)=x
            x=x+d
         enddo
!     function set
         iopt(1)=3
         iopt(2)=3
         call SPLC(pairx, mesh_pair, pairy, DF, IOPT, pairc, mesh_pair, IER)
         if (iii==1) then
            call SPLF(pairx,mesh_pair,pairy,pairc,mesh_pair,pairxn,mesh,pairTi, IER)
            call SPLD(pairx,mesh_pair,pairc,mesh_pair,pairxn,mesh,pairdTi,spd2, IER)
         else if (iii==2) then
            call SPLF(pairx,mesh_pair,pairy,pairc,mesh_pair,pairxn,mesh,pairAl, IER)
            call SPLD(pairx,mesh_pair,pairc,mesh_pair,pairxn,mesh,pairdAl,spd2, IER)
         else
            call SPLF(pairx,mesh_pair,pairy,pairc,mesh_pair,pairxn,mesh,pairTial, IER)
            call SPLD(pairx,mesh_pair,pairc,mesh_pair,pairxn,mesh,pairdTial,spd2, IER)
         endif
         deallocate (pairxn,spd2)
         deallocate(pairx,pairy,pairc)
      enddo


! density function
      do iii=1,2
         if (iii==1) open (10,file='pot/EAMPARAM.PRB68-024102.ti.den')
         if (iii==2) then
            if (alonly) open (10,file='pot/EAMPARAM.PRB68-024102-Al.al.den')
            if (.not.alonly) open (10,file='pot/EAMPARAM.PRB68-024102.al.den')
         endif
         mesh_den=0
 110     read (10,*,end=130) ichar !trial read
         if (ichar(:1)=='#') then
            go to 110           !skip a line if it is a comment line
         else
            backspace 10        !read again if it is not a comment line
         endif
 120     read (10,*,end=130,err=110) a1,a2
         mesh_den=mesh_den+1
         go to 120
 130     close (10)
         allocate(denx(mesh_den),deny(mesh_den),denc(mesh_den,3))

         if (iii==1) open (10,file='pot/EAMPARAM.PRB68-024102.ti.den')
         if (iii==2) then
            if (alonly) open (10,file='pot/EAMPARAM.PRB68-024102-Al.al.den')
            if (.not.alonly) open (10,file='pot/EAMPARAM.PRB68-024102.al.den')
         endif
         mesh_den=0
 115     read (10,*,end=135) ichar !trial read
         if (ichar(:1)=='#') then
            go to 115           !skip a line if it is a comment line
         else
            backspace 10        !read again if it is not a comment line
         endif
 125     read (10,*,end=135,err=115) a1,a2
         mesh_den=mesh_den+1
         if (mesh_den==1) then
            if (iii==1) denminTi=a1
            if (iii==2) denminAl=a1
         endif
         denx(mesh_den)=a1
         deny(mesh_den)=a2
         go to 125
 135     close (10)
         if (iii==1) denmaxTi=a1
         if (iii==2) denmaxAl=a1
!     finer mesh set
         allocate(denxn(mesh),spd2(mesh))
         if (iii==1) then
            x=denminTi
            d=(denmaxTi-denminTi)/dble(mesh-1)
         else
            x=denminAl
            d=(denmaxAl-denminAl)/dble(mesh-1)
         endif
         do i=1,mesh
            denxn(i)=x
            x=x+d
         enddo
!     function set
         iopt(1)=3
         iopt(2)=3
         call SPLC(denx, mesh_den, deny, DF, IOPT, denc, mesh_den, IER)
         if (iii==1) then
            call SPLF(denx,mesh_den,deny,denc,mesh_den,denxn,mesh,denTi, IER)
            call SPLD(denx,mesh_den,denc,mesh_den,denxn,mesh,dendTi,spd2, IER)
         else
            call SPLF(denx,mesh_den,deny,denc,mesh_den,denxn,mesh,denAl, IER)
            call SPLD(denx,mesh_den,denc,mesh_den,denxn,mesh,dendAl,spd2, IER)
         endif
         deallocate (denxn,spd2)
         deallocate(denx,deny,denc)
      enddo

! Embed function
      do iii=1,2
         if (iii==1) open (10,file='pot/EAMPARAM.PRB68-024102.ti.embed')
         if (iii==2) then
            if (alonly) open (10,file='pot/EAMPARAM.PRB68-024102-Al.al.embed')
            if (.not.alonly) open (10,file='pot/EAMPARAM.PRB68-024102.al.embed')
         endif
         mesh_embed=0
 210     read (10,*,end=230) ichar !trial read
         if (ichar(:1)=='#') then
            go to 210           !skip a line if it is a comment line
         else
            backspace 10        !read again if it is not a comment line
         endif
 220     read (10,*,end=230,err=210) a1,a2
         mesh_embed=mesh_embed+1
         go to 220
 230     close (10)
         allocate(embedx(mesh_embed),embedy(mesh_embed),embedc(mesh_embed,3))

         if (iii==1) open (10,file='pot/EAMPARAM.PRB68-024102.ti.embed')
         if (iii==2) then
            if (alonly) open (10,file='pot/EAMPARAM.PRB68-024102-Al.al.embed')
            if (.not.alonly) open (10,file='pot/EAMPARAM.PRB68-024102.al.embed')
         endif
         mesh_embed=0
 215     read (10,*,end=235) ichar !trial read
         if (ichar(:1)=='#') then
            go to 215           !skip a line if it is a comment line
         else
            backspace 10        !read again if it is not a comment line
         endif
 225     read (10,*,end=235,err=215) a1,a2
         mesh_embed=mesh_embed+1
         if (mesh_embed==1) then
            if (iii==1) embedminTi=a1
            if (iii==2) embedminAl=a1
         endif
         embedx(mesh_embed)=a1
         embedy(mesh_embed)=a2
         go to 225
 235     close (10)
         if (iii==1) embedmaxTi=a1
         if (iii==2) embedmaxAl=a1
!     finer mesh set
         allocate(embedxn(mesh),spd2(mesh))
         if (iii==1) then
            x=embedminTi
            d=(embedmaxTi-embedminTi)/dble(mesh-1)
         else
            x=embedminAl
            d=(embedmaxAl-embedminAl)/dble(mesh-1)
         endif
         do i=1,mesh
            embedxn(i)=x
            x=x+d
         enddo
!     function set
         iopt(1)=3
         iopt(2)=3
         call SPLC(embedx, mesh_embed, embedy, DF, IOPT, embedc, mesh_embed, IER)
         if (iii==1) then
            call SPLF(embedx,mesh_embed,embedy,embedc,mesh_embed,embedxn,mesh,embedTi, IER)
            call SPLD(embedx,mesh_embed,embedc,mesh_embed,embedxn,mesh,embeddTi,spd2, IER)
         else
            call SPLF(embedx,mesh_embed,embedy,embedc,mesh_embed,embedxn,mesh,embedAl, IER)
            call SPLD(embedx,mesh_embed,embedc,mesh_embed,embedxn,mesh,embeddAl,spd2, IER)
         endif
         deallocate (embedxn,spd2)
         deallocate(embedx,embedy,embedc)
      enddo

      eam_tial_setup_done=.true.

      end

      
